@@ -274,7 +274,6 @@ void* Migrate::worker_main(void *data)
 
 void Migrate::start(const SpiceMsgMainMigrationBegin* migrate)
 {
-    std::string cert_subject;
     uint32_t peer_major;
     uint32_t peer_minor;
 
@@ -1312,7 +1311,7 @@ void RedClient::dispatch_agent_message(VDAgentMessage* msg, void* data)
     default:
         DBG(0, "Unsupported message type %u size %u", msg->type, msg->size);
     }
-}                                       
+}
 
 void RedClient::handle_agent_tokens(RedPeer::InMessage* message)
 {
@@ -1408,4 +1407,3 @@ void RedClient::register_channel_factory(ChannelFactory& factory)
 {
     _factorys.push_back(&factory);
 }
-
