@@ -23,6 +23,10 @@
 #endif
 
 #include <assert.h>
+#include <errno.h>
+#include <string.h>
+#include <netinet/in.h> // IPPROTO_TCP
+#include <netinet/tcp.h> // TCP_NODELAY
 
 #include "server/char_device.h"
 #include "server/red_channel.h"
@@ -82,6 +86,19 @@ static void spicevmc_chardev_wakeup(SpiceCharDeviceInstance *sin)
 
 static int spicevmc_red_channel_client_config_socket(RedChannelClient *rcc)
 {
+    int delay_val = 1;
+    RedsStream *stream = red_channel_client_get_stream(rcc);
+
+    if (rcc->channel->type == SPICE_CHANNEL_USBREDIR) {
+        if (setsockopt(stream->socket, IPPROTO_TCP, TCP_NODELAY,
+                &delay_val, sizeof(delay_val)) != 0) {
+            if (errno != ENOTSUP) {
+                red_printf("setsockopt failed, %s", strerror(errno));
+                return FALSE;
+            }
+        }
+    }
+
     return TRUE;
 }
 
@@ -99,7 +116,11 @@ static void spicevmc_red_channel_client_on_disconnect(RedChannelClient *rcc)
     sin = state->chardev_sin;
     sif = SPICE_CONTAINEROF(sin->base.sif, SpiceCharDeviceInterface, base);
 
-    red_channel_client_destroy(rcc);
+    /* Don't destroy the rcc if the entire client is disconnecting, as then
+       red_client_destroy will already do this! */
+    if (!rcc->client->disconnecting)
+        red_channel_client_destroy(rcc);
+
     state->rcc = NULL;
     if (sif->state) {
         sif->state(sin, 0);
@@ -107,7 +128,9 @@ static void spicevmc_red_channel_client_on_disconnect(RedChannelClient *rcc)
 }
 
 static int spicevmc_red_channel_client_handle_message(RedChannelClient *rcc,
-    SpiceDataHeader *header, uint8_t *msg)
+                                                      uint16_t type,
+                                                      uint32_t size,
+                                                      uint8_t *msg)
 {
     SpiceVmcState *state;
     SpiceCharDeviceInstance *sin;
@@ -117,22 +140,22 @@ static int spicevmc_red_channel_client_handle_message(RedChannelClient *rcc,
     sin = state->chardev_sin;
     sif = SPICE_CONTAINEROF(sin->base.sif, SpiceCharDeviceInterface, base);
 
-    if (header->type != SPICE_MSGC_SPICEVMC_DATA) {
-        return red_channel_client_handle_message(rcc, header->size,
-                                                 header->type, msg);
+    if (type != SPICE_MSGC_SPICEVMC_DATA) {
+        return red_channel_client_handle_message(rcc, size, type, msg);
     }
 
     /*
      * qemu spicevmc will consume everything we give it, no need for
      * flow control checks (or to use a pipe).
      */
-    sif->write(sin, msg, header->size);
+    sif->write(sin, msg, size);
 
     return TRUE;
 }
 
 static uint8_t *spicevmc_red_channel_alloc_msg_rcv_buf(RedChannelClient *rcc,
-    SpiceDataHeader *msg_header)
+                                                       uint16_t type,
+                                                       uint32_t size)
 {
     SpiceVmcState *state;
 
@@ -140,9 +163,9 @@ static uint8_t *spicevmc_red_channel_alloc_msg_rcv_buf(RedChannelClient *rcc,
 
     assert(!state->rcv_buf_in_use);
 
-    if (msg_header->size > state->rcv_buf_size) {
-        state->rcv_buf = spice_realloc(state->rcv_buf, msg_header->size);
-        state->rcv_buf_size = msg_header->size;
+    if (size > state->rcv_buf_size) {
+        state->rcv_buf = spice_realloc(state->rcv_buf, size);
+        state->rcv_buf_size = size;
     }
 
     state->rcv_buf_in_use = 1;
@@ -151,7 +174,9 @@ static uint8_t *spicevmc_red_channel_alloc_msg_rcv_buf(RedChannelClient *rcc,
 }
 
 static void spicevmc_red_channel_release_msg_rcv_buf(RedChannelClient *rcc,
-    SpiceDataHeader *msg_header, uint8_t *msg)
+                                                     uint16_t type,
+                                                     uint32_t size,
+                                                     uint8_t *msg)
 {
     SpiceVmcState *state;
 
